=== modified file 'checkbox/application.py'
--- checkbox/application.py	2009-12-06 18:07:40 +0000
+++ checkbox/application.py	2010-02-11 22:28:33 +0000
@@ -85,6 +85,10 @@
                           type="string",
                           default=[],
                           help=_("Configuration override parameters."))
+        parser.add_option("-b", "--blacklist",
+                          help=_("Shorthand for --config=checkbox/plugins/jobs_prompt/blacklist."))
+        parser.add_option("-w", "--whitelist",
+                          help=_("Shorthand for --config=checkbox/plugins/jobs_prompt/whitelist."))
         return parser.parse_args(args)
 
     def create_application(self, args=sys.argv):
@@ -97,6 +101,13 @@
         args[:0] = split(string_options)
         (options, args) = self.parse_options(args)
 
+        # Replace shorthands
+        for shorthand in "blacklist", "whitelist":
+            key = "checkbox/plugins/job_prompt/%s" % shorthand
+            value = getattr(options, shorthand)
+            if value:
+                options.config.append("=".join(key, value))
+
         # Set logging early
         set_logging(options.log_level, options.log)
 

=== modified file 'checkbox/reactor.py'
--- checkbox/reactor.py	2010-02-10 00:40:13 +0000
+++ checkbox/reactor.py	2010-02-11 22:28:33 +0000
@@ -46,14 +46,6 @@
         self._event_handlers = {}
         self._local = threading.local()
 
-    @property
-    def _event_stack(self):
-        try:
-            return self._local._event_stack
-        except AttributeError:
-            self._local._event_stack = []
-            return self._local._event_stack
-
     def call_on(self, event_type, handler, priority=0):
         pair = (handler, priority)
 
@@ -66,8 +58,6 @@
     def fire(self, event_type, *args, **kwargs):
         logging.debug("Started firing %s.", event_type)
 
-        self._event_stack.append(event_type)
-
         results = []
         handlers = []
         for key, value in self._event_handlers.items():
@@ -99,8 +89,6 @@
                                   format_object(handler), event_type,
                                   args, kwargs)
 
-        self._event_stack.pop(-1)
-
         logging.debug("Finished firing %s.", event_type)
         return results
 

=== modified file 'checkbox/user_interface.py'
--- checkbox/user_interface.py	2010-02-10 00:38:31 +0000
+++ checkbox/user_interface.py	2010-02-11 22:28:33 +0000
@@ -69,31 +69,6 @@
         self.gettext_domain = "checkbox"
         gettext.textdomain(self.gettext_domain)
 
-    def _toggle_results(self, key, options, results):
-        if isinstance(results, dict):
-            if key in results:
-                del results[key]
-
-            elif key in options:
-                if isinstance(options[key], dict):
-                    results[key] = {}
-                elif isinstance(options[key], (list, tuple,)):
-                    results[key] = []
-                else:
-                    raise Exception, "Unknown result type: %s" % type(results)
-
-                for k in options[key]:
-                    self._toggle_results(k, options[key], results[key])
-
-        elif isinstance(results, (list, tuple,)):
-            if key in results:
-                results.remove(key)
-            elif key in options:
-                results.append(key)
-
-        else:
-            raise Exception, "Unknown result type: %s" % type(results)
-
     def show_info(self, text, options=[], default=None):
         logging.info(text)
         return default

=== modified file 'checkbox_cli/cli_interface.py'
--- checkbox_cli/cli_interface.py	2010-02-10 00:38:31 +0000
+++ checkbox_cli/cli_interface.py	2010-02-11 22:28:33 +0000
@@ -195,6 +195,31 @@
 
 class CLIInterface(UserInterface):
 
+    def _toggle_results(self, key, options, results):
+        if isinstance(results, dict):
+            if key in results:
+                del results[key]
+
+            elif key in options:
+                if isinstance(options[key], dict):
+                    results[key] = {}
+                elif isinstance(options[key], (list, tuple,)):
+                    results[key] = []
+                else:
+                    raise Exception, "Unknown result type: %s" % type(results)
+
+                for k in options[key]:
+                    self._toggle_results(k, options[key], results[key])
+
+        elif isinstance(results, (list, tuple,)):
+            if key in results:
+                results.remove(key)
+            elif key in options:
+                results.append(key)
+
+        else:
+            raise Exception, "Unknown result type: %s" % type(results)
+
     def show_progress_start(self, text):
         self.progress = CLIProgressDialog(text)
         self.progress.show()
@@ -260,6 +285,9 @@
                 response = dialog.get()
                 do_expand = True
 
+            else:
+                do_expand = False
+
             # Invalid response
             if response < 0:
                 continue
@@ -270,19 +298,13 @@
                 self._toggle_results(result, options, results)
                 continue
 
+            # Expand tree
             dialog.visible = False
-            if result not in results:
-                self._toggle_results(result, options, results)
-
-            # Expand tree or check
-            value = values[response]
-            if isinstance(value, dict):
-                self.show_tree(result, options[result], results[result])
-
-            elif isinstance(value, (list, tuple,)):
-                self.show_check(result, options[result], results[result])
-
-            do_expand = False
+            if options[result]:
+                branch_results =  results.get(result, {})
+                self.show_tree(result, options[result], branch_results)
+                if branch_results and result not in results:
+                    results[result] = branch_results
 
         return results
 

=== modified file 'checkbox_gtk/gtk_interface.py'
--- checkbox_gtk/gtk_interface.py	2010-02-10 00:38:31 +0000
+++ checkbox_gtk/gtk_interface.py	2010-02-11 22:28:33 +0000
@@ -189,25 +189,6 @@
 
         return state
 
-    def _set_treestore(self, treestore, children, defaults, parent=None):
-        if type(children) == dict:
-            if type(defaults) != dict:
-                defaults = {}
-            for text, child in children.items():
-                active = text in defaults
-                iter = treestore.append(parent, [text, active, children, defaults])
-                self._set_treestore(treestore, child, defaults.get(text), iter)
-
-        elif type(children) in (list, tuple):
-            if type(defaults) not in (list, tuple):
-                defaults = []
-            for text in children:
-                active = text in defaults
-                treestore.append(parent, [text, active, children, defaults])
-
-        else:
-            raise Exception, "Unknown children type: %s" % type(children)
-
     def _run_dialog(self, dialog=None):
         def on_dialog_response(dialog, response, self):
             # Keep dialog alive when the button that has been clicked
@@ -363,7 +344,7 @@
 
     @GTKHack
     def show_tree(self, text, options={}, default={}):
-        (COLUMN_TEXT, COLUMN_ACTIVE, COLUMN_OPTIONS, COLUMN_RESULTS) = range(4)
+        (COLUMN_TEXT, COLUMN_ACTIVE) = range(2)
 
         # Set buttons
         self._notebook.set_current_page(1)
@@ -371,54 +352,54 @@
         self._set_show("button_deselect_all", True)
 
         # Set options
-        treestore = gtk.TreeStore(str, bool, object, object)
-        self._set_treestore(treestore, options, default)
-
+        treestore = gtk.TreeStore(str, bool)
         treeview = gtk.TreeView(treestore)
         treeview.set_headers_visible(False)
         treeview.show()
 
+        def set_options(options, default, parent=None):
+            keys = sorted(options.keys())
+            values = [options[k] for k in keys]
+            for text, child in zip(keys, values):
+                active = text in default
+                iter = treestore.append(parent, [text, active])
+                set_options(child, default.get(text, {}), iter)
+
+        set_options(options, default)
+
         vbox = self._get_widget("vbox_options_list")
         vbox.pack_start(treeview, False, False, 0)
 
-        # Toggle boxes
-        model = treeview.get_model()
-
-        def toggle_results(toggle, path, is_expand=False):
-            if isinstance(path, str):
-                iter = model.get_iter_from_string(path)
-            else:
-                iter = model.get_iter(path)
-
-            # Toggle results
-            key = model.get_value(iter, COLUMN_TEXT)
-            options = model.get_value(iter, COLUMN_OPTIONS)
-            results = model.get_value(iter, COLUMN_RESULTS)
-
-            if is_expand and key in results:
-                return
-
-            self._toggle_results(key, options, results)
-
+        def toggle_tree(iter, active=None):
             # Set value
-            active = not model.get_value(iter, COLUMN_ACTIVE)
-            model.set_value(iter, COLUMN_ACTIVE, active)
+            if active is None:
+                active = not treestore.get_value(iter, COLUMN_ACTIVE)
+            treestore.set_value(iter, COLUMN_ACTIVE, active)
+
+            # Set parents
+            def set_parents(iter):
+                parent = treestore.iter_parent(iter)
+                if parent:
+                    treestore.set_value(parent, COLUMN_ACTIVE, active)
+                    set_parents(parent)
+
+            if active:
+                set_parents(iter)
 
             # Set children
-            sibling = model.iter_children(iter)
-            while sibling:
-                model.set_value(sibling, COLUMN_ACTIVE, active)
-                if results and key in results:
-                    model.set_value(sibling, COLUMN_RESULTS, results[key])
-                sibling = model.iter_next(sibling)
+            def set_children(iter):
+                child = treestore.iter_children(iter)
+                while child:
+                    treestore.set_value(child, COLUMN_ACTIVE, active)
+                    set_children(child)
+                    child = treestore.iter_next(child)
 
-        # Make sure interfaces behave the same way
-        treeview.connect("row-expanded",
-            lambda tv, iter, path: toggle_results(iter, path, True))
+            set_children(iter)
 
         # Toggle column
         cell = gtk.CellRendererToggle()
-        cell.connect("toggled", toggle_results)
+        cell.connect("toggled",
+            lambda t, p: toggle_tree(treestore.get_iter_from_string(p)))
         col = gtk.TreeViewColumn(None, cell, active=COLUMN_ACTIVE)
         treeview.append_column(col)
 
@@ -429,13 +410,10 @@
 
         # Select and deselect buttons
         def click_button(widget, active):
-            sibling = model.get_iter_first()
-            while sibling:
-                if model.get_value(sibling, COLUMN_ACTIVE) != active:
-                    path = model.get_path(sibling)
-                    toggle_results(None, path)
-
-                sibling = model.iter_next(sibling)
+            iter = treestore.get_iter_first()
+            while iter:
+                toggle_tree(iter, active)
+                iter = treestore.iter_next(iter)
 
         for button_name in "button_select_all", "button_deselect_all":
             button = self._get_widget(button_name)
@@ -449,9 +427,23 @@
         self._set_show("button_deselect_all", False)
 
         # Get options
+        def get_results(iter):
+            results = {}
+            while iter:
+                active = treestore.get_value(iter, COLUMN_ACTIVE)
+                if active:
+                    text = treestore.get_value(iter, COLUMN_TEXT)
+                    children = treestore.iter_children(iter)
+                    results[text] = get_results(children)
+
+                iter = treestore.iter_next(iter)
+
+            return results
+
+        results = get_results(treestore.get_iter_first())
         vbox.remove(treeview)
 
-        return default
+        return results
 
     def _run_test(self, test):
         self._set_sensitive("button_test", False)

=== modified file 'debian/checkbox.config'
--- debian/checkbox.config	2010-01-04 20:59:37 +0000
+++ debian/checkbox.config	2010-02-11 22:28:33 +0000
@@ -6,14 +6,12 @@
     configure)
         db_input medium checkbox/plugins/apport_prompt/default_enabled || true
         db_input medium checkbox/plugins/apport_prompt/default_package || true
+        db_input medium checkbox/plugins/jobs_prompt/blacklist || true
+        db_input medium checkbox/plugins/jobs_prompt/whitelist || true
         db_input medium checkbox/plugins/launchpad_exchange/transport_url || true
         db_input medium checkbox/plugins/launchpad_prompt/email || true
         db_input medium checkbox/plugins/proxy_info/http_proxy || true
         db_input medium checkbox/plugins/proxy_info/https_proxy || true
-        db_input medium checkbox/plugins/suites_prompt/blacklist || true
-        db_input medium checkbox/plugins/suites_prompt/whitelist || true
-        db_input medium checkbox/plugins/tests_prompt/blacklist || true
-        db_input medium checkbox/plugins/tests_prompt/whitelist || true
         db_go
         ;;
     *)

=== modified file 'debian/checkbox.install'
--- debian/checkbox.install	2009-10-12 15:46:11 +0000
+++ debian/checkbox.install	2010-02-11 22:28:33 +0000
@@ -6,13 +6,12 @@
 usr/share/checkbox/configs
 usr/share/checkbox/data/*
 usr/share/checkbox/install/*
+usr/share/checkbox/jobs/*
 usr/share/checkbox/patches/*
 usr/share/checkbox/plugins/*
 usr/share/checkbox/registries/*
 usr/share/checkbox/report/*
 usr/share/checkbox/scripts/*
-usr/share/checkbox/suites/*
-usr/share/checkbox/tests/*
 usr/share/checkbox/examples/checkbox.ini
 usr/share/checkbox/examples/checkbox-backend.ini
 usr/share/checkbox/examples/checkbox-backend.ini etc/checkbox.d/

=== modified file 'debian/checkbox.templates'
--- debian/checkbox.templates	2010-01-04 20:59:37 +0000
+++ debian/checkbox.templates	2010-02-11 22:28:33 +0000
@@ -8,6 +8,16 @@
 _Description: Default Package
  Default package if none is detected.
 
+Template: checkbox/plugins/jobs_prompt/blacklist
+Type: string
+_Description: Suites Blacklist
+ List of jobs to blacklist
+
+Template: checkbox/plugins/jobs_prompt/whitelist
+Type: string
+_Description: Suites Whitelist
+ List of jobs to whitelist
+
 Template: checkbox/plugins/launchpad_exchange/transport_url
 Type: string
 Default: https://launchpad.net/+hwdb/+submit
@@ -28,24 +38,3 @@
 Type: string
 _Description: HTTPS Proxy
  HTTPS proxy to use instead of the one specified in environment.
-
-Template: checkbox/plugins/suites_prompt/blacklist
-Type: string
-_Description: Suites Blacklist
- List of suites to blacklist
-
-Template: checkbox/plugins/suites_prompt/whitelist
-Type: string
-_Description: Suites Whitelist
- List of suites to whitelist
-
-Template: checkbox/plugins/tests_prompt/blacklist
-Type: string
-_Description: Tests Blacklist
- List of tests to blacklist
-
-Template: checkbox/plugins/tests_prompt/whitelist
-Type: string
-_Description: Tests Whitelist
- List of tests to whitelist
-

=== modified file 'examples/checkbox.ini'
--- examples/checkbox.ini	2010-01-21 20:22:38 +0000
+++ examples/checkbox.ini	2010-02-11 22:28:33 +0000
@@ -31,12 +31,8 @@
 
 blacklist = hal
 
-[checkbox/plugins/suites_prompt]
+[checkbox/plugins/jobs_prompt]
 
-# Blacklist of suites that are potentially dangerous
+# Blacklist of jobs that are potentially dangerous
 blacklist = autotest ltp phoronix qa-regression-testing
-
-[checkbox/plugins/tests_prompt]
-
-# Blacklist of tests that are known to fail
-blacklist = clone06.1 ioctl03.1 lchown02.* swapon03.1
+  clone06.1 ioctl03.1 lchown02.* swapon03.1

=== renamed directory 'tests' => 'jobs'
=== renamed file 'suites/autotest.txt.in' => 'jobs/autotest.txt.in'
=== modified file 'jobs/disk.txt.in'
--- tests/disk.txt.in	2010-01-04 16:43:24 +0000
+++ jobs/disk.txt.in	2010-02-11 22:28:33 +0000
@@ -1,5 +1,5 @@
 plugin: manual
-name: disk
+name: disk_test
 timeout: 10
 requires: package.name == 'lshw'
 user: root

=== renamed file 'suites/internal.txt.in' => 'jobs/internal.txt.in'
--- suites/internal.txt.in	2010-02-04 20:28:10 +0000
+++ jobs/internal.txt.in	2010-02-11 22:28:33 +0000
@@ -1,42 +1,52 @@
 name: audio
 plugin: internal
 _description: Audio tests
-command: cat $CHECKBOX_SHARE/tests/audio.txt*
+command: cat $CHECKBOX_SHARE/jobs/audio.txt*
 
 name: disk
 plugin: internal
 _description: Disk tests
-command: cat $CHECKBOX_SHARE/tests/disk.txt*
+command: cat $CHECKBOX_SHARE/jobs/disk.txt*
 
 name: fingerprint
 plugin: internal
 _description: Fingerprint reader tests
-command: cat $CHECKBOX_SHARE/tests/fingerprint.txt*
+command: cat $CHECKBOX_SHARE/jobs/fingerprint.txt*
 
 name: firewire
 plugin: internal
 _description: Firewire disk tests
-command: cat $CHECKBOX_SHARE/tests/firewire.txt*
+command: cat $CHECKBOX_SHARE/jobs/firewire.txt*
 
 name: kms
 plugin: internal
 _description: Kernel modesetting tests
-command: cat $CHECKBOX_SHARE/tests/kms.txt*
+command: cat $CHECKBOX_SHARE/jobs/kms.txt*
+
+name: media
+plugin: internal
+_description: Media tests
+command: cat $CHECKBOX_SHARE/jobs/media.txt*
+
+name: monitor
+plugin: internal
+_description: Monitor tests
+command: cat $CHECKBOX_SHARE/jobs/monitor.txt*
 
 name: network
 plugin: internal
 _description: Network tests
-command: cat $CHECKBOX_SHARE/tests/network.txt*
+command: cat $CHECKBOX_SHARE/jobs/network.txt*
 
 name: peripheral
 plugin: internal
 _description: Peripheral tests
-command: cat $CHECKBOX_SHARE/tests/peripheral.txt*
+command: cat $CHECKBOX_SHARE/jobs/peripheral.txt*
 
 name: video
 plugin: internal
 _description: Video tests
-command: cat $CHECKBOX_SHARE/tests/video.txt*
+command: cat $CHECKBOX_SHARE/jobs/video.txt*
 
 name: codecs
 plugin: attachment

=== renamed file 'suites/ltp.txt.in' => 'jobs/ltp.txt.in'
=== modified file 'jobs/network.txt.in'
--- tests/network.txt.in	2009-10-27 14:30:35 +0000
+++ jobs/network.txt.in	2010-02-11 22:28:33 +0000
@@ -1,5 +1,5 @@
 plugin: manual
-name: network
+name: network_test
 requires: device.category == 'NETWORK'
 command: network_test
 _description:
@@ -10,7 +10,7 @@
  Is this correct?
 
 plugin: manual
-name: internet
+name: internet_test
 depends: network
 command: internet_test
 _description:
@@ -21,7 +21,7 @@
  Is this correct?
 
 plugin: manual
-name: modem
+name: modem_test
 command: network_check
 _description:
  Built-in modem network connection procedure:

=== renamed file 'suites/phoronix.txt.in' => 'jobs/phoronix.txt.in'
=== renamed file 'suites/qa_regression.txt.in' => 'jobs/qa_regression.txt.in'
=== modified file 'jobs/video.txt.in'
--- tests/video.txt.in	2010-02-05 20:36:07 +0000
+++ jobs/video.txt.in	2010-02-11 22:28:33 +0000
@@ -11,7 +11,7 @@
  Is this acceptable for your display?
 
 plugin: manual
-name: video
+name: display
 requires: package.name == 'python-gst0.10'
 command: gst_pipeline_test -t 2 'videotestsrc ! ffmpegcolorspace ! gconfvideosink'
 _description:

=== modified file 'plugins/external_suite.py'
--- plugins/external_suite.py	2009-07-28 15:56:22 +0000
+++ plugins/external_suite.py	2010-02-11 22:28:33 +0000
@@ -30,7 +30,13 @@
             self._manager.reactor.call_on(rt, rh)
 
     def prompt_external(self, interface, suite):
+        def report_job(job):
+            job["suite"] = suite["name"]
+
+        event_id = self._manager.reactor.call_on("report-job", report_job)
         self._manager.reactor.fire("message-exec", suite)
+        self._manager.reactor.cancel_call(event_id)
+
         self._manager.reactor.fire("prompt-tests", interface)
 
     def report_external(self, suite):

=== modified file 'plugins/internal_suite.py'
--- plugins/internal_suite.py	2010-02-10 00:40:13 +0000
+++ plugins/internal_suite.py	2010-02-11 22:28:33 +0000
@@ -44,7 +44,13 @@
 
     def report_internal(self, suite):
         self._manager.reactor.fire("report-suite", suite)
+
+        def report_job(job):
+            job["suite"] = suite["name"]
+
+        event_id = self._manager.reactor.call_on("report-job", report_job)
         self._manager.reactor.fire("message-exec", suite)
+        self._manager.reactor.cancel_call(event_id)
 
 
 factory = InternalSuite

=== added file 'plugins/jobs_prompt.py'
--- plugins/jobs_prompt.py	1970-01-01 00:00:00 +0000
+++ plugins/jobs_prompt.py	2010-02-11 22:28:33 +0000
@@ -0,0 +1,84 @@
+#
+# This file is part of Checkbox.
+#
+# Copyright 2008 Canonical Ltd.
+#
+# Checkbox is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Checkbox is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
+#
+import re
+
+from checkbox.job import UNINITIATED
+from checkbox.properties import List, Path, String
+from checkbox.plugin import Plugin
+
+
+class JobsPrompt(Plugin):
+
+    # Plugin default for running job types
+    plugin_default = String(default="external")
+
+    # Status default for jobs
+    status_default = String(default=UNINITIATED)
+
+    # Space separated list of directories where job files are stored.
+    directories = List(Path(),
+        default_factory=lambda:"%(checkbox_share)s/jobs")
+
+    # List of jobs to blacklist
+    blacklist = List(String(), default_factory=lambda:"")
+
+    # List of jobs to whitelist
+    whitelist = List(String(), default_factory=lambda:"")
+
+    def register(self, manager):
+        super(JobsPrompt, self).register(manager)
+        self._iterator = None
+        self._jobs = {}
+
+        for (rt, rh) in [
+             ("gather", self.gather),
+             ("report-job", self.report_job),
+             ("prompt-job", self.prompt_job)]:
+            self._manager.reactor.call_on(rt, rh)
+
+    def gather(self):
+        for directory in self.directories:
+            self._manager.reactor.fire("message-directory", directory)
+
+    def report_job(self, job):
+        whitelist_patterns = [re.compile(r"^%s$" % r) for r in self.whitelist if r]
+        blacklist_patterns = [re.compile(r"^%s$" % r) for r in self.blacklist if r]
+
+        # Stop if job not in whitelist or in blacklist
+        name = job["name"]
+        if whitelist_patterns:
+            if not [name for p in whitelist_patterns if p.match(name)]:
+                self._manager.reactor.stop()
+        elif blacklist_patterns:
+            if [name for p in blacklist_patterns if p.match(name)]:
+                self._manager.reactor.stop()
+
+        # Update job
+        job.update(self._jobs.get(name, {}))
+        job.setdefault("plugin", self.plugin_default)
+        job.setdefault("status", self.status_default)
+        self._jobs[name] = job
+
+        self._manager.reactor.fire("report-%s" % job["plugin"], job)
+
+    def prompt_job(self, interface, job):
+        self._manager.reactor.fire("prompt-%s" % job["plugin"], interface, job)
+
+
+factory = JobsPrompt

=== modified file 'plugins/message_info.py'
--- plugins/message_info.py	2009-10-12 15:46:11 +0000
+++ plugins/message_info.py	2010-02-11 22:28:33 +0000
@@ -64,7 +64,7 @@
 
     @coerce_arguments(message=message_schema)
     def message(self, message):
-        self._manager.reactor.fire("report-%s" % message["plugin"], message)
+        self._manager.reactor.fire("report-job", message)
 
     def messages(self, messages):
         for message in messages:

=== modified file 'plugins/subunit_report.py'
--- plugins/subunit_report.py	2009-07-22 18:02:57 +0000
+++ plugins/subunit_report.py	2010-02-11 22:28:33 +0000
@@ -47,9 +47,8 @@
             self._manager.reactor.call_on(rt, rh)
 
     def report_test(self, test):
-        key = (test["suite"], test["name"],)
-        if key not in self._tests:
-            self._tests[key] = test
+        name = test["name"]
+        self._tests[name] = test
 
     def report(self):
         logging.debug("Opening filename: %s", self.filename)

=== modified file 'plugins/suites_prompt.py'
--- plugins/suites_prompt.py	2010-02-10 00:40:13 +0000
+++ plugins/suites_prompt.py	2010-02-11 22:28:33 +0000
@@ -16,12 +16,13 @@
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 #
-import re
+import copy
 
 from checkbox.lib.iterator import IteratorExclude
+from checkbox.lib.resolver import Resolver
 
 from checkbox.job import JobIterator
-from checkbox.properties import List, Path, String
+from checkbox.properties import List, String
 from checkbox.plugin import Plugin
 
 from gettext import gettext as _
@@ -29,121 +30,88 @@
 
 class SuitesPrompt(Plugin):
 
-    # Plugin default for running suite types
-    plugin_default = String(default="external")
-
-    # Plugin priorities for running suites
+    # Plugin priorities for running jobs
     plugin_priorities = List(String(), default_factory=lambda:"internal")
 
-    # Space separated list of directories where suite files are stored.
-    directories = List(Path(),
-        default_factory=lambda:"%(checkbox_share)s/suites")
-
-    # List of suites to blacklist
-    blacklist = List(String(), default_factory=lambda:"")
-
-    # List of suites to whitelist
-    whitelist = List(String(), default_factory=lambda:"")
-
-    def _suites_compare(self, a, b):
-        priorities = self.plugin_priorities
-        if a["plugin"] in priorities:
-            if b["plugin"] in priorities:
-                ia = priorities.index(a["plugin"])
-                ib = priorities.index(b["plugin"])
-                if ia != ib:
-                    return cmp(ia, ib)
-            else:
-                return -1
-        elif b["plugin"] in priorities:
-            return 1
-
-        return cmp(a["name"], b["name"])
-
-    def _suites_exclude(self, suite):
-        suites_ignore = self.persist.get("ignore", [])
-        if "description" in suite and suite["description"] in suites_ignore:
-            return True
-
-        return False
-
     def register(self, manager):
         super(SuitesPrompt, self).register(manager)
         self._iterator = None
-        self._suite = None
+        self._ignore = []
         self._suites = {}
         self._tests = {}
 
         for (rt, rh) in [
-             ("gather", self.gather),
              ("gather-persist", self.gather_persist),
+             ("ignore-jobs", self.ignore_jobs),
              ("report-suite", self.report_suite),
-             ("prompt-suite", self.prompt_suite),
+             ("report-test", self.report_test),
              ("prompt-suites", self.prompt_suites)]:
             self._manager.reactor.call_on(rt, rh)
 
         self._manager.reactor.call_on("prompt-gather", self.prompt_gather, 100)
-        self._manager.reactor.call_on("report-(attachment|test)",
-            self.report_attachment_or_test, -100)
-
-    def gather(self):
-        for directory in self.directories:
-            self._manager.reactor.fire("message-directory", directory)
 
     def gather_persist(self, persist):
         self.persist = persist.root_at("suites_prompt")
 
-    def report_attachment_or_test(self, element):
-        if self._suite:
-            element.setdefault("suite", self._suite["name"])
-            if element["plugin"] != "attachment":
-                self._tests.setdefault(self._suite["name"], [])
-                self._tests[self._suite["name"]].append(element["name"])
+    def ignore_jobs(self, jobs):
+        self._ignore = jobs
 
     def report_suite(self, suite):
-        whitelist_patterns = [re.compile(r"^%s$" % r) for r in self.whitelist if r]
-        blacklist_patterns = [re.compile(r"^%s$" % r) for r in self.blacklist if r]
-
         name = suite["name"]
-        if whitelist_patterns:
-            if not [name for p in whitelist_patterns if p.match(name)]:
-                return
-        elif blacklist_patterns:
-            if [name for p in blacklist_patterns if p.match(name)]:
-                return
+        suite.update(self._suites.get(name, {}))
+        suite.setdefault("type", "suite")
+        self._suites[name] = suite
 
-        if suite["name"] not in self._suites:
-            suite.setdefault("plugin", self.plugin_default)
-            self._suites[suite["name"]] = self._suite = suite
+    def report_test(self, test):
+        name = test["name"]
+        self._tests[name] = test
 
     def prompt_gather(self, interface):
-        if len(self._suites) > 1:
-            suites_tree = dict((s["description"], self._tests.get(s["name"], []))
-                for s in self._suites.values() if "description" in s)
-            suites_default = self.persist.get("default")
-            if suites_default is None:
-                suites_default = dict(suites_tree)
-
-            suites_results = interface.show_tree(
-                _("Select the suites to test"),
-                suites_tree, suites_default)
-            self.persist.set("default", suites_results)
-
-            ignore_tests = []
-            for suite, tests in suites_tree.items():
-                if suite not in suites_results:
-                    ignore_tests.extend(tests)
-
-                else:
-                    for test in tests:
-                        if test not in suites_results[suite]:
-                            ignore_tests.append(test)
-
-            self._manager.reactor.fire("ignore-tests", ignore_tests)
-
-    def prompt_suite(self, interface, suite):
-        self._manager.reactor.fire("prompt-%s" % suite["plugin"],
-            interface, suite)
+        # Resolve dependencies
+        compare = lambda a, b: cmp(a["name"], b["name"])
+        resolver = Resolver(compare=compare, key=lambda e: e["name"])
+        jobs = dict((k, v) for k, v in self._suites.items() + self._tests.items())
+        for job in jobs.values():
+            depends = [jobs[job["suite"]]] if "suite" in job else []
+            resolver.add(job, *depends)
+
+        # Build options
+        options = {}
+        for job in resolver.get_dependents():
+            suboptions = options
+            dependencies = resolver.get_dependencies(job)
+            for dependency in dependencies:
+                if dependency.get("type") == "suite":
+                    attribute = "description"
+                else:
+                    attribute = "name"
+
+                suboptions = suboptions.setdefault(dependency[attribute], {})
+
+        # Builds defaults
+        defaults = self.persist.get("default")
+        if defaults is None:
+            defaults = copy.deepcopy(options)
+
+        # Get results
+        results = interface.show_tree(_("Select the suites to test"),
+            options, defaults)
+        self.persist.set("default", results)
+
+        # Get tests to ignore
+        def get_ignore_jobs(options, results):
+            jobs = []
+            for k, v in options.iteritems():
+                if not v and k not in results:
+                    jobs.append(k)
+
+                else:
+                    jobs.extend(get_ignore_jobs(options[k], results.get(k, {})))
+
+            return jobs
+
+        ignore_jobs = get_ignore_jobs(options, results)
+        self._manager.reactor.fire("ignore-jobs", ignore_jobs)
 
     def prompt_suites(self, interface):
         if not self._iterator:
@@ -154,11 +122,32 @@
 
         while True:
             try:
-                self._suite = self._iterator.go(interface.direction)
+                suite = self._iterator.go(interface.direction)
             except StopIteration:
                 break
 
-            self._manager.reactor.fire("prompt-suite", interface, self._suite)
+            self._manager.reactor.fire("prompt-job", interface, suite)
+
+    def _suites_compare(self, a, b):
+        priorities = self.plugin_priorities
+        if a["plugin"] in priorities:
+            if b["plugin"] in priorities:
+                ia = priorities.index(a["plugin"])
+                ib = priorities.index(b["plugin"])
+                if ia != ib:
+                    return cmp(ia, ib)
+            else:
+                return -1
+        elif b["plugin"] in priorities:
+            return 1
+
+        return cmp(a["name"], b["name"])
+
+    def _suites_exclude(self, suite):
+        if "description" in suite and suite["description"] in self._ignore:
+            return True
+
+        return False
 
 
 factory = SuitesPrompt

=== modified file 'plugins/tests_prompt.py'
--- plugins/tests_prompt.py	2010-02-10 00:40:13 +0000
+++ plugins/tests_prompt.py	2010-02-11 22:28:33 +0000
@@ -16,29 +16,14 @@
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 #
-import re
-
 from checkbox.lib.iterator import IteratorExclude, PREV
 
-from checkbox.job import JobIterator, UNINITIATED
+from checkbox.job import JobIterator
 from checkbox.plugin import Plugin
-from checkbox.properties import List, String
 
 
 class TestsPrompt(Plugin):
 
-    # List of suites to blacklist
-    blacklist = List(String(), default_factory=lambda:"")
-
-    # List of suites to whitelist
-    whitelist = List(String(), default_factory=lambda:"")
-
-    def _tests_exclude(self, test):
-        if test["name"] in self._ignore:
-            return True
-
-        return False
-
     def register(self, manager):
         super(TestsPrompt, self).register(manager)
         self._iterator = None
@@ -46,10 +31,9 @@
         self._tests = {}
 
         for (rt, rh) in [
+             ("ignore-jobs", self.ignore_jobs),
              ("report", self.report),
              ("report-test", self.report_test),
-             ("ignore-tests", self.ignore_tests),
-             ("prompt-test", self.prompt_test),
              ("prompt-tests", self.prompt_tests)]:
             self._manager.reactor.call_on(rt, rh)
 
@@ -57,29 +41,13 @@
         self._manager.reactor.fire("report-tests", self._tests.values())
 
     def report_test(self, test):
-        whitelist_patterns = [re.compile(r"^%s$" % r) for r in self.whitelist if r]
-        blacklist_patterns = [re.compile(r"^%s$" % r) for r in self.blacklist if r]
-
         name = test["name"]
-        if whitelist_patterns:
-            if not [name for p in whitelist_patterns if p.match(name)]:
-                return
-        elif blacklist_patterns:
-            if [name for p in blacklist_patterns if p.match(name)]:
-                return
-
-        key = (test["suite"], test["name"],)
-        if key not in self._tests:
-            test.setdefault("type", "test")
-            test.setdefault("status", UNINITIATED)
-            self._tests[key] = test
-
-    def ignore_tests(self, tests):
-        self._ignore = tests
-
-    def prompt_test(self, interface, test):
-        self._manager.reactor.fire("prompt-%s" % test["plugin"],
-            interface, test)
+        test.update(self._tests.get(name, {}))
+        test.setdefault("type", "test")
+        self._tests[name] = test
+
+    def ignore_jobs(self, jobs):
+        self._ignore = jobs
 
     def prompt_tests(self, interface, blacklist=[], whitelist=[]):
         # TODO: blacklist and whitelist are being ignored
@@ -97,7 +65,13 @@
             except StopIteration:
                 break
 
-            self._manager.reactor.fire("prompt-test", interface, test)
+            self._manager.reactor.fire("prompt-job", interface, test)
+
+    def _tests_exclude(self, test):
+        if test["name"] in self._ignore:
+            return True
+
+        return False
 
 
 factory = TestsPrompt

=== modified file 'setup.cfg'
--- setup.cfg	2010-01-21 20:21:25 +0000
+++ setup.cfg	2010-02-11 22:28:33 +0000
@@ -6,19 +6,18 @@
 desktop_files=[
     ("share/applications", ("gtk/checkbox-gtk.desktop.in",))]
 rfc822deb_files=[
-    ("share/checkbox/suites",
-        ("suites/autotest.txt.in",
-         "suites/internal.txt.in",
-         "suites/ltp.txt.in",
-         "suites/phoronix.txt.in",
-         "suites/qa_regression.txt.in",)),
-    ("share/checkbox/tests",
-        ("tests/audio.txt.in",
-         "tests/disk.txt.in",
-         "tests/fingerprint.txt.in",
-         "tests/firewire.txt.in",
-         "tests/kms.txt.in",
-         "tests/media.txt.in",
-         "tests/network.txt.in",
-         "tests/peripheral.txt.in",
-         "tests/video.txt.in",))]
+    ("share/checkbox/jobs",
+        ("jobs/audio.txt.in",
+         "jobs/autotest.txt.in",
+         "jobs/disk.txt.in",
+         "jobs/fingerprint.txt.in",
+         "jobs/firewire.txt.in",
+         "jobs/internal.txt.in",
+         "jobs/kms.txt.in",
+         "jobs/ltp.txt.in",
+         "jobs/media.txt.in",
+         "jobs/network.txt.in",
+         "jobs/peripheral.txt.in",
+         "jobs/phoronix.txt.in",
+         "jobs/qa_regression.txt.in",
+         "jobs/video.txt.in",))]

=== removed directory 'suites'
